-- constants
local BeastCleaveSpellId = 118455;
local FrenzySpellId = 272790;
local BindingShotSpellId = 109248;
local MultiShotSpellId = 2643;
local TarTrapSpellId = 187698;
local TarTrapDebuffSpellId = 135299;
local FlareSpellId = 1543;
local NoPetErrorMessageId = 52;
local BeastCleaveName = GetSpellInfo(BeastCleaveSpellId);
local FrenzyName = GetSpellInfo(FrenzySpellId);
--local MultiShotName = GetSpellInfo(MultiShotSpellId);
--local TarTrapName = GetSpellInfo(TarTrapSpellId);
--local FlareName = GetSpellInfo(FlareSpellId);
local NoPetErrorMessage = "You do not have a pet.";
local NoPetErrorMessageIsDead = "Your pet is dead.";
-- local copies of functions called many times
local select = select;
local UnitExists, UnitIsDead, FindAuraByName, CombatLogGetCurrentEventInfo = UnitExists, UnitIsDead, AuraUtil.FindAuraByName, CombatLogGetCurrentEventInfo;
-- Created later
local SBCTSaves, BCTFrame, DFFrame, TTFrame, TTFrameUpdater, PetAlertFrame;
local PlayerGUID;

local function SetDefaultPosistion(player)
	BCTFrame:ClearAllPoints();
	if(player == nil) then
		BCTFrame:SetPoint("LEFT",PlayerFrame,"RIGHT");
	else
		BCTFrame:SetPoint("LEFT",PetFrame,"RIGHT",-4,0);
	end

	DFFrame:ClearAllPoints();
	DFFrame:SetPoint("LEFT",BCTFrame,"RIGHT",4,0);
	
	TTFrame:ClearAllPoints();
	TTFrame:SetPoint("LEFT",DFFrame,"RIGHT",4,0);

	BCTFrame:SetUserPlaced(true);
	DFFrame:SetUserPlaced(true);
	TTFrame:SetUserPlaced(true);
end

local function SetFrameSize(size)
	BCTFrame:SetSize(size, size);
	DFFrame:SetSize(size, size);
	TTFrame:SetSize(size, size);
	_G[BCTOptions.size:GetName().."Text"]:SetText(size);
end

local function SetFrameStrata(strata)
	BCTFrame:SetFrameStrata(strata);
	DFFrame:SetFrameStrata(strata);
	TTFrame:SetFrameStrata(strata);
	UIDropDownMenu_SetText(BCTOptions.frameStrata, strata);
end

local function HideCooldownText(value)
	BCTFrame.cooldown:SetHideCountdownNumbers(value);
	DFFrame.cooldown:SetHideCountdownNumbers(value);
	TTFrame.cooldown:SetHideCountdownNumbers(value);
end

local function AlertPetMissing()
	if(not UnitExists("pet")) then
		PetAlertFrame:AddMessage("|TInterface\\Icons\\ability_hunter_beastcall:0|t No active Pet |TInterface\\Icons\\ability_hunter_beastcall:0|t", 1, 0, 0);
	elseif(UnitIsDead("pet")) then
		PetAlertFrame:AddMessage("|TInterface\\Icons\\ability_hunter_beastsoothe:0|t Pet is Dead |TInterface\\Icons\\ability_hunter_beastsoothe:0|t", 1, 0, 0);
	end
end

local function AnnounceBinding()
	local msg = "<BCT> Used my |cff71d5ff|Hspell:"..BindingShotSpellId..":0|h[Binding Shot]|h|r up for 10 sec.";
	if(IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) then
		SendChatMessage(msg, "INSTANCE_CHAT");
	elseif(IsInGroup(LE_PARTY_CATEGORY_HOME)) then
		SendChatMessage(msg, "PARTY");
	else
		print(msg);
	end
end

-- Pet Alert Frame
PetAlertFrame = CreateFrame("MessageFrame","BeastCleaveTrackerPetAlertFrame",UIParent);
PetAlertFrame:Show();
PetAlertFrame:SetInsertMode("TOP");
PetAlertFrame:SetFrameStrata("DIALOG");
PetAlertFrame:SetSize(800,35);
PetAlertFrame:SetPoint("BOTTOM",UIParent,"CENTER");
PetAlertFrame:SetFontObject("SubZoneTextFont");
PetAlertFrame:SetTimeVisible(1);
PetAlertFrame:SetFadeDuration(1);

-- Beast Cleave
BCTFrame = CreateFrame("Frame","BeastCleaveTracker",UIParent);
BCTFrame:Hide();
BCTFrame:SetMovable(true);
BCTFrame:EnableMouse(false);
BCTFrame:RegisterForDrag("LeftButton");
BCTFrame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
BCTFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); end);

BCTFrame.texture = BCTFrame:CreateTexture(nil,"ARTWORK");
BCTFrame.texture:SetAllPoints(BCTFrame);
BCTFrame.texture:SetTexture("Interface\\Icons\\ability_hunter_sickem");

BCTFrame.cooldown = CreateFrame("Cooldown", nil, BCTFrame, "CooldownFrameTemplate");
BCTFrame.cooldown:SetAllPoints(BCTFrame);
BCTFrame.cooldown:SetDrawBling(false);
BCTFrame.cooldown:SetDrawSwipe(true);
BCTFrame.cooldown:SetDrawEdge(true);

-- Frenzy
DFFrame = CreateFrame("Frame","BeastCleaveTrackerDireFrenzy",UIParent);
DFFrame:Hide();
DFFrame:SetMovable(true);
DFFrame:EnableMouse(false);
DFFrame:RegisterForDrag("LeftButton");
DFFrame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
DFFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); end);

DFFrame.texture = DFFrame:CreateTexture(nil,"ARTWORK");
DFFrame.texture:SetAllPoints(DFFrame);
DFFrame.texture:SetTexture("Interface\\Icons\\ability_hunter_barbedshot");

DFFrame.cooldown = CreateFrame("Cooldown", nil, DFFrame, "CooldownFrameTemplate");
DFFrame.cooldown:SetAllPoints(DFFrame);
DFFrame.cooldown:SetDrawBling(false);
DFFrame.cooldown:SetDrawSwipe(true);
DFFrame.cooldown:SetDrawEdge(true);

DFFrame.stackCount = DFFrame:CreateFontString(nil, nil, "TextStatusBarTextLarge");
DFFrame.stackCount:SetPoint("BOTTOMRIGHT", DFFrame, "BOTTOMRIGHT", 1, -1);

-- Tar Trap
TTFrame = CreateFrame("Frame","BeastCleaveTrackerTarTrap",UIParent);
TTFrame:Hide();
TTFrame:SetMovable(true);
TTFrame:EnableMouse(false);
TTFrame:RegisterForDrag("LeftButton");
TTFrame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
TTFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); end);

TTFrame.texture = TTFrame:CreateTexture(nil,"ARTWORK");
TTFrame.texture:SetAllPoints(TTFrame);
TTFrame.texture:SetTexture("Interface\\Icons\\spell_yorsahj_bloodboil_black");

TTFrame.cooldown = CreateFrame("Cooldown", nil, TTFrame, "CooldownFrameTemplate");
TTFrame.cooldown:SetAllPoints(TTFrame);
TTFrame.cooldown:SetDrawBling(false);
TTFrame.cooldown:SetDrawSwipe(true);
TTFrame.cooldown:SetDrawEdge(true);
TTFrame.cooldown:SetScript("OnHide", function() if(not BCTFrame.movingMode) then TTFrame:Hide(); end end);

TTFrame.flareCD = TTFrame:CreateFontString(nil, nil, "TextStatusBarTextLarge");
TTFrame.flareCD:SetPoint("BOTTOMRIGHT", TTFrame, "BOTTOMRIGHT", 1, -1);
TTFrame.flareCD:SetText("F");

-- Tar Trap Updater
TTFrameUpdater = CreateFrame("Frame","BeastCleaveTrackerTarTrapUpdater",UIParent);
TTFrameUpdater:Hide();
TTFrameUpdater.start = 0;
TTFrameUpdater.updateAllowed = false;
TTFrameUpdater.newCast = false;
TTFrameUpdater.lastUpdate = 0;
TTFrameUpdater:SetScript("OnUpdate", function(self, elapsed)
	self.lastUpdate = self.lastUpdate + elapsed;
	if(self.lastUpdate > 0.3) then 
		-- Update 3 times a second
		self.lastUpdate = 0;
		local t = GetTime();
		-- handle flare cooldown
		local _,d = GetSpellCooldown(FlareSpellId);
		if(d > 2) then
			TTFrame.flareCD:SetTextColor(1,0,0);
		else
			TTFrame.flareCD:SetTextColor(0,1,0);
		end
		-- allow time to be updated by aura apply
		if(self.updateAllowed and t > self.start + 3) then
			self.updateAllowed = false;
		end
		-- autostart timer after this duration
		if(self.updateAllowed and self.newCast and t > self.start + 1.5) then
			self.newCast = false;
			TTFrame:Show();
			TTFrame.cooldown:SetCooldown(GetTime(),30);
		end
		-- no need to run update if no tar trap is out
		if(t > self.start + 35) then
			self:Hide();
		end
	end
end);

-- Events
SetDefaultPosistion();

local function onevent(self, event, ...)
	--if(string.find(event,"SPELL")) then
	--	print(event,...)
	--end
	if(event == "COMBAT_LOG_EVENT_UNFILTERED") then
		local t, e, _, sguid, s, _, _, dguid, d, _, _, spellId, spellName  = CombatLogGetCurrentEventInfo();
		--print(CombatLogGetCurrentEventInfo());
		if(e == "SPELL_CAST_SUCCESS" and sguid == PlayerGUID) then
			if(SBCTSaves.announceBinding and spellId == BindingShotSpellId) then
				AnnounceBinding();
			elseif(SBCTSaves.alertPet and spellId == MultiShotSpellId) then
				AlertPetMissing();
			elseif(SBCTSaves.tarTrap and spellId == TarTrapSpellId) then
				TTFrameUpdater.start = GetTime();
				TTFrameUpdater.newCast = true;
				TTFrameUpdater.updateAllowed = true;
				TTFrameUpdater:Show();
				if(not BCTFrame.movingMode) then
					TTFrame:Hide();
				end
			end
		elseif((e == "SPELL_AURA_APPLIED" or e == "SPELL_AURA_REFRESH") and sguid == PlayerGUID) then
			if(TTFrameUpdater.updateAllowed and spellId == TarTrapDebuffSpellId) then
				TTFrameUpdater.updateAllowed = false;
				TTFrame:Show();
				TTFrame.cooldown:SetCooldown(GetTime(),30);
			end
		end
	elseif(event == "UNIT_AURA") then
		local u = ...;
		if(u == "pet") then
			-- beast cleave
			local n,_,c,_,d,e = FindAuraByName(BeastCleaveName, "pet", "HELPFUL");
			if(n) then
				BCTFrame:Show();
				BCTFrame.cooldown:SetCooldown(e-d,d);
			elseif(not BCTFrame.movingMode) then
				BCTFrame:Hide();
			end
			
			-- frenzy
			local n,_,c,_,d,e = FindAuraByName(FrenzyName, "pet", "HELPFUL");
			if(SBCTSaves.direFrenzy and n) then
				DFFrame:Show();
				DFFrame.cooldown:SetCooldown(e-d,d);
				if(c > 1) then
					DFFrame.stackCount:SetText(c);
					DFFrame.stackCount:Show();
					if(c > 2) then DFFrame.stackCount:SetTextColor(0,1,0); else DFFrame.stackCount:SetTextColor(1,1,1); end
				else
					DFFrame.stackCount:Hide();
				end
			elseif(not BCTFrame.movingMode) then
				DFFrame:Hide();
			end
		end
	elseif(event == "UI_ERROR_MESSAGE") then
		local id, msg = ...;
		if(SBCTSaves.alertPet and id == NoPetErrorMessageId and (msg == NoPetErrorMessage or msg == NoPetErrorMessageIsDead)) then
			AlertPetMissing();
		end
	elseif(event == "ADDON_LOADED") then
		local name = ...;
		if(name == "SimpleBeastCleaveTracker") then
			-- saved vars
			if(_G["SBCTSaves"] == nil) then _G["SBCTSaves"] = {}; end
			SBCTSaves = _G["SBCTSaves"];
			if(SBCTSaves.size == nil) then SBCTSaves.size = 30; end
			if(SBCTSaves.strata == nil) then SBCTSaves.strata = "MEDIUM"; end
			if(SBCTSaves.hideText == nil) then SBCTSaves.hideText = false; end
			if(SBCTSaves.announceBinding == nil) then SBCTSaves.announceBinding = false; end
			if(SBCTSaves.alertPet == nil) then SBCTSaves.alertPet = true; end
			if(SBCTSaves.direFrenzy == nil) then SBCTSaves.direFrenzy = true; end
			if(SBCTSaves.tarTrap == nil) then SBCTSaves.tarTrap = true; end
			-- restore from saved
			SetFrameSize(SBCTSaves.size);
			SetFrameStrata(SBCTSaves.strata);
			HideCooldownText(SBCTSaves.hideText);
			-- setup options frame
			BCTOptions.size:SetValue(SBCTSaves.size);
			BCTOptions.hideText:SetChecked(SBCTSaves.hideText);
			BCTOptions.direFrenzy:SetChecked(SBCTSaves.direFrenzy);
			BCTOptions.tarTrap:SetChecked(SBCTSaves.tarTrap);
			BCTOptions.alertPet:SetChecked(SBCTSaves.alertPet);
			BCTOptions.announceBinding:SetChecked(SBCTSaves.announceBinding);
			-- register events
			BCTFrame:UnregisterEvent("ADDON_LOADED");
			BCTFrame:RegisterEvent("UNIT_AURA");
			BCTFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
			BCTFrame:RegisterEvent("UI_ERROR_MESSAGE");
			BCTFrame:RegisterEvent("PLAYER_TALENT_UPDATE");
			--BCTFrame:RegisterAllEvents();
		end
	elseif(event == "PLAYER_LOGIN") then
		PlayerGUID = UnitGUID("player");
	end
end

-- **************
-- Option Frame
-- **************

local onEnterTooltipHelper = function(self)
	if(self.tooltipText ~= nil) then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		GameTooltip:SetText(self.tooltipText);
	end
end

local onLeaveTooltipHelper = function(self)
	if(self.tooltipText ~= nil) then
		GameTooltip:Hide();
	end
end

local function setupTooltip(self, text)
	self.tooltipText = text;
	self:SetScript("OnEnter", onEnterTooltipHelper);
	self:SetScript("OnLeave", onLeaveTooltipHelper);
end

local BCTOptions = CreateFrame("Frame","BCTOptions",UIParent);
BCTOptions.name = "Beast Cleave Tracker";
InterfaceOptions_AddCategory(BCTOptions);

BCTOptions.title = BCTOptions:CreateFontString();
BCTOptions.title:SetFontObject("GameFontNormalLarge");
BCTOptions.title:SetText("Simple Beast Cleave Tracker");
BCTOptions.title:SetPoint("TOPLEFT",BCTOptions, "TOPLEFT", 16, -16);

BCTOptions.move = CreateFrame("CheckButton", "BCTOptionsMove", BCTOptions, "UICheckButtonTemplate");
BCTOptions.move:SetPoint("TOPLEFT",BCTOptions.title, "BOTTOMLEFT");
_G[BCTOptions.move:GetName() .. "Text"]:SetText("Unlock frame for moving");
setupTooltip(BCTOptions.move, "You can drag the icons freely when checked.");
BCTOptions.move:SetScript("OnClick", function(self)
	if(self:GetChecked()) then
		BCTFrame:Show();
		BCTFrame:EnableMouse(true);
		BCTFrame.movingMode = true;
		DFFrame:Show();
		DFFrame:EnableMouse(true);
		TTFrame:Show();
		TTFrame:EnableMouse(true);
	else
		BCTFrame:EnableMouse(false);
		BCTFrame:Hide();
		BCTFrame.movingMode = false;
		DFFrame:EnableMouse(false);
		DFFrame:Hide();
		TTFrame:EnableMouse(false);
		TTFrame:Hide();
	end
end);

BCTOptions.moveReset = CreateFrame("Button", "BCTOptionsMoveReset", BCTOptions, "UIPanelButtonTemplate");
BCTOptions.moveReset:SetPoint("TOPLEFT",BCTOptions.move, "BOTTOMLEFT");
BCTOptions.moveReset:SetText("Reset to player frame");
BCTOptions.moveReset.tooltipText = "Reset icons positions to player frame."
BCTOptions.moveReset:SetWidth(200);
BCTOptions.moveReset:SetScript("OnClick", function() SetDefaultPosistion(); end);

BCTOptions.moveResetPet = CreateFrame("Button", "BCTOptionsMoveResetPet", BCTOptions, "UIPanelButtonTemplate");
BCTOptions.moveResetPet:SetPoint("TOPLEFT",BCTOptions.moveReset, "BOTTOMLEFT");
BCTOptions.moveResetPet:SetText("Reset to pet frame");
BCTOptions.moveResetPet.tooltipText = "Reset icons positions to pet frame."
BCTOptions.moveResetPet:SetWidth(200);
BCTOptions.moveResetPet:SetScript("OnClick", function() SetDefaultPosistion(true); end);

BCTOptions.sizeText = BCTOptions:CreateFontString();
BCTOptions.sizeText:SetFontObject("GameFontNormal");
BCTOptions.sizeText:SetText("Size (default: 30)");
BCTOptions.sizeText:SetPoint("TOPLEFT", BCTOptions.moveResetPet, "BOTTOMLEFT", 0, -16);

BCTOptions.size = CreateFrame("Slider", "BCTOptionsSize", BCTOptions, "OptionsSliderTemplate");
BCTOptions.size:SetPoint("TOPLEFT",BCTOptions.sizeText, "BOTTOMLEFT", 0, -16);
BCTOptions.size:SetWidth(200);
BCTOptions.size:SetMinMaxValues(4, 80);
BCTOptions.size:SetValueStep(2);
BCTOptions.size:SetObeyStepOnDrag(true); 
_G[BCTOptions.size:GetName().."High"]:SetText(80);
_G[BCTOptions.size:GetName().."Low"]:SetText(4);
BCTOptions.size:SetScript("OnValueChanged", function(self, value) SBCTSaves.size = value; SetFrameSize(SBCTSaves.size); end);

BCTOptions.frameStrataText = BCTOptions:CreateFontString();
BCTOptions.frameStrataText:SetFontObject("GameFontNormal");
BCTOptions.frameStrataText:SetText("Layer Strata (default: Medium)");
BCTOptions.frameStrataText:SetPoint("TOPLEFT", BCTOptions.size, "BOTTOMLEFT", 0, -24);

BCTOptions.frameStrata = CreateFrame("Frame", "BCTOptionsFrameStrata", BCTOptions, "UIDropDownMenuTemplate");
BCTOptions.frameStrata:SetPoint("TOPLEFT", BCTOptions.frameStrataText, "BOTTOMLEFT", 0, -8);
UIDropDownMenu_Initialize(BCTOptions.frameStrata, function(frame, level, menuList)
	local info = UIDropDownMenu_CreateInfo();
	info.func = function(self, value) SBCTSaves.strata = value; SetFrameStrata(SBCTSaves.strata) end;
	info.notCheckable = true;
	info.text, info.arg1 = "Background", "BACKGROUND";
	UIDropDownMenu_AddButton(info);
	info.text, info.arg1 = "Low", "LOW";
	UIDropDownMenu_AddButton(info);
	info.text, info.arg1 = "Medium", "MEDIUM";
	UIDropDownMenu_AddButton(info);
	info.text, info.arg1 = "High", "HIGH";
	UIDropDownMenu_AddButton(info);
	info.text, info.arg1 = "Dialog", "DIALOG";
	UIDropDownMenu_AddButton(info);
end);

BCTOptions.hideText = CreateFrame("CheckButton", "BCTOptionsHideText", BCTOptions, "UICheckButtonTemplate");
BCTOptions.hideText:SetPoint("TOPLEFT", BCTOptions.frameStrata, "BOTTOMLEFT");
_G[BCTOptions.hideText:GetName() .. "Text"]:SetText("Hide cooldown text");
setupTooltip(BCTOptions.hideText, "Default the cooldown text will mimic what you have set for blizzard action bars,\nif you never want to show it check this.");
BCTOptions.hideText:SetScript("OnClick", function(self) if(self:GetChecked()) then SBCTSaves.hideText = true; else SBCTSaves.hideText = false; end HideCooldownText(SBCTSaves.hideText) end);

BCTOptions.direFrenzy = CreateFrame("CheckButton", "BCTOptionsDireFenzy", BCTOptions, "UICheckButtonTemplate");
BCTOptions.direFrenzy:SetPoint("TOPLEFT", BCTOptions.hideText, "BOTTOMLEFT");
_G[BCTOptions.direFrenzy:GetName() .. "Text"]:SetText("Track Frenzy");
setupTooltip(BCTOptions.direFrenzy, "Enables the Frenzy tracking icon.");
BCTOptions.direFrenzy:SetScript("OnClick", function(self) if(self:GetChecked()) then SBCTSaves.direFrenzy = true; else SBCTSaves.direFrenzy = false; end end);

BCTOptions.tarTrap = CreateFrame("CheckButton", "BCTOptionsTarTrap", BCTOptions, "UICheckButtonTemplate");
BCTOptions.tarTrap:SetPoint("TOPLEFT", BCTOptions.direFrenzy, "BOTTOMLEFT");
_G[BCTOptions.tarTrap:GetName() .. "Text"]:SetText("Track Tar Trap");
setupTooltip(BCTOptions.tarTrap, "Enables the Tar Trap duration icon. A small F will change color depending on if Flair is ready.");
BCTOptions.tarTrap:SetScript("OnClick", function(self) if(self:GetChecked()) then SBCTSaves.tarTrap = true; else SBCTSaves.tarTrap = false; end end);

BCTOptions.alertPet = CreateFrame("CheckButton", "BCTOptionsAlertPet", BCTOptions, "UICheckButtonTemplate");
BCTOptions.alertPet:SetPoint("TOPLEFT", BCTOptions.tarTrap, "BOTTOMLEFT");
_G[BCTOptions.alertPet:GetName() .. "Text"]:SetText("Pet Reminder");
setupTooltip(BCTOptions.alertPet, "Will alert you if you are shooting multi-shots,\nor trying to use kill command and are missing a pet.");
BCTOptions.alertPet:SetScript("OnClick", function(self) if(self:GetChecked()) then SBCTSaves.alertPet = true; else SBCTSaves.alertPet = false; end end);

BCTOptions.announceBinding = CreateFrame("CheckButton", "BCTOptionsAnnounceBinding", BCTOptions, "UICheckButtonTemplate");
BCTOptions.announceBinding:SetPoint("TOPLEFT", BCTOptions.alertPet, "BOTTOMLEFT");
_G[BCTOptions.announceBinding:GetName() .. "Text"]:SetText("Announce Binding Shot");
setupTooltip(BCTOptions.announceBinding, "Lets your party know when you are using Binding Shot.\nAnnounces in instance chat or party chat, not raid chat.");
BCTOptions.announceBinding:SetScript("OnClick", function(self) if(self:GetChecked()) then SBCTSaves.announceBinding = true; else SBCTSaves.announceBinding = false; end end);

BCTFrame:RegisterEvent("ADDON_LOADED");
BCTFrame:RegisterEvent("PLAYER_LOGIN");
BCTFrame:SetScript("OnEvent", onevent);